import DeleteIcon from "@mui/icons-material/DeleteOutlined";
import DriveFileMoveOutlineIcon from "@mui/icons-material/DriveFileMoveOutline";
import { Tooltip, Typography } from "@mui/material";
import { DataGrid, GridActionsCellItem, GridColDef } from "@mui/x-data-grid";
import React from "react";
import { useParams } from "react-router-dom";
import { Movement, MovementApi } from "../api/MovementsApi";
import { useAccounts } from "../hooks/AccountsListProvider";
import { useAlert } from "../hooks/context_providers/AlertDialogProvider";
import { useSelectAccount } from "../hooks/context_providers/ChooseAccountDialogProvider";
import { useConfirm } from "../hooks/context_providers/ConfirmDialogProvider";
import { useLoadingMessage } from "../hooks/context_providers/LoadingMessageProvider";
import { useSnackbar } from "../hooks/context_providers/SnackbarProvider";
import { AccountWidget } from "../widgets/AccountWidget";
import { AmountWidget } from "../widgets/AmountWidget";
import { AsyncWidget } from "../widgets/AsyncWidget";
import { DateWidget } from "../widgets/DateWidget";
import { MoneyMgrWebRouteContainer } from "../widgets/MoneyMgrWebRouteContainer";
import { NewMovementWidget } from "../widgets/NewMovementWidget";
import { NotFoundRoute } from "./NotFound";

export function AccountRoute(): React.ReactElement {
  const loadingMessage = useLoadingMessage();

  const { accountId } = useParams();

  const loadKey = React.useRef(0);

  const accounts = useAccounts();
  const account = accounts.get(Number(accountId));

  const [movements, setMovements] = React.useState<Movement[] | undefined>();

  const load = async () => {
    setMovements(await MovementApi.GetAccountMovements(account!.id));
  };

  const reload = async (skipMovements = false) => {
    try {
      accounts.reloadBalances();
      if (!skipMovements) {
        loadingMessage.show("Refreshing the list of movements");
        await load();
      }
    } catch (e) {
      console.error("Failed to load list of movements!", e);
      alert(`Failed to refresh the list of movements! ${e}`);
    } finally {
      loadingMessage.hide();
    }
  };

  if (account === null) return <NotFoundRoute />;

  return (
    <MoneyMgrWebRouteContainer
      label={
        <span style={{ display: "inline-flex", alignItems: "center" }}>
          <AccountWidget account={account} />
          &nbsp;
          <span style={{ display: "inline-flex", flexDirection: "column" }}>
            <span>{account.name}</span>
            <Typography component={"span"} variant="subtitle1">
              <AmountWidget amount={account.balance} />
            </Typography>
          </span>
        </span>
      }
    >
      <div style={{ display: "flex", flexDirection: "column", flex: 1 }}>
        <div style={{ flex: 1 }}>
          <AsyncWidget
            loadKey={`${account.id}-${loadKey.current}`}
            load={load}
            ready={movements !== null}
            errMsg="Failed to load the list of movements!"
            build={() => (
              <MovementsTable needReload={reload} movements={movements!} />
            )}
          />
        </div>
        <NewMovementWidget account={account} onCreated={reload} />
      </div>
    </MoneyMgrWebRouteContainer>
  );
}

function MovementsTable(p: {
  movements: Movement[];
  needReload: (skipMovements: boolean) => void;
}): React.ReactElement {
  const accounts = useAccounts();
  const alert = useAlert();
  const confirm = useConfirm();
  const snackbar = useSnackbar();

  const chooseAccount = useSelectAccount();

  // Change account of movement
  const handleMoveClick = async (movement: Movement) => {
    const target = await chooseAccount(
      "Transfer movement",
      `Please select the target account that will receive the movement: ${movement.label} (${movement.amount} €)`,
      "Transfer movement",
      [accounts.get(movement.account_id)!]
    );

    if (!target) return;

    try {
      movement.account_id = target.id;

      await MovementApi.Update(movement);

      snackbar("The movement has been successfully moved!");

      p.needReload(false);
    } catch (e) {
      console.error("Failed to update movement information!", e);
      alert(`Failed to change movemlent account! ${e}`);
    }
  };

  // Delete movement
  const handleDeleteClick = async (movement: Movement) => {
    try {
      if (
        !(await confirm(
          `Do you really want to delete the movement ${movement.label} (${movement.amount}€)?`
        ))
      )
        return;

      await MovementApi.Delete(movement);

      const id = p.movements.findIndex((m) => movement.id === m.id);
      p.movements.slice(id, id);

      p.needReload(false);
    } catch (e) {
      console.error("Failed to delete movement!", e);
      alert(`Failed to delete movement! ${e}`);
    }
  };

  const columns: GridColDef<(typeof p.movements)[number]>[] = [
    {
      field: "checked",
      headerName: "Checked",
      width: 50,
      type: "boolean",
      editable: true,
    },
    {
      field: "time",
      headerName: "Date",
      width: 98 + 80,
      editable: true,
      type: "dateTime",
      valueGetter(_, m) {
        return new Date(m.time * 1000);
      },
      valueSetter(v, row) {
        row.time = Math.floor(v.getTime() / 1000);
        return row;
      },
      renderCell: (params) => {
        return <DateWidget time={params.row.time} />;
      },
    },
    {
      field: "label",
      headerName: "Label",
      flex: 1,
      editable: true,
      type: "string",
    },
    {
      field: "amount",
      headerName: "Amount",
      width: 110,
      editable: true,
      type: "number",
      align: "left",
      headerAlign: "left",
      renderCell: (params) => {
        return <AmountWidget amount={params.row.amount} />;
      },
    },
    {
      field: "file",
      headerName: "File",
      // TODO
    },
    {
      field: "actions",
      type: "actions",
      headerName: "Actions",
      width: 80,
      cellClassName: "actions",
      editable: false,
      getActions: (params) => {
        return [
          <Tooltip title="Move to another account">
            <GridActionsCellItem
              icon={<DriveFileMoveOutlineIcon />}
              label="Move to another account"
              onClick={() => handleMoveClick(params.row)}
              color="inherit"
            />
          </Tooltip>,
          <Tooltip title="Delete the movement">
            <GridActionsCellItem
              icon={<DeleteIcon color="error" />}
              label="Delete"
              onClick={() => handleDeleteClick(params.row)}
              color="inherit"
            />
          </Tooltip>,
        ];
      },
    },
  ];
  return (
    <DataGrid<Movement>
      columns={columns}
      rows={p.movements}
      autoPageSize
      checkboxSelection
      initialState={{
        sorting: {
          sortModel: [{ field: "time", sort: "desc" }],
        },
        columns: {
          columnVisibilityModel: {
            checked: false,
          },
        },
      }}
      processRowUpdate={async (n) => {
        try {
          return await MovementApi.Update(n);
        } catch (e) {
          console.error("Failed to update movement information!", e);
          alert(`Failed to update row! ${e}`);
          throw e;
        } finally {
          p.needReload(true);
        }
      }}
    />
  );
}
